<?php

class Dbmc_Sanitization_Helper {

	/**
	 * The name of this plugin
	 * @var string
	 */
	private $dbmc;

	/**
	 * The array containing plugin settings
	 * @var array
	 */
	private $registered_settings;

	/**
	 * Initialize the class and set its properties
	 * @param string $dbmc The name of this plugin
	 */
	public function __construct( $dbmc ) {

		$this->dbmc = $dbmc;
		$this->registered_settings = Dbmc_Settings_Definition::get_settings();

		add_filter( 'dbmc_settings_sanitize_text', array( $this, 'sanitize_text_field' ) );
		add_filter( 'dbmc_settings_sanitize_email', array( $this, 'sanitize_email_field' ) );
		add_filter( 'dbmc_settings_sanitize_checkbox', array( $this, 'sanitize_checkbox_field' ) );
		add_filter( 'dbmc_settings_sanitize_url', array( $this, 'sanitize_url_field' ) );
	
	}

	/**
	 * Settings sanitization
	 *
	 * Adds a settings error (for the updated message)
	 * At some point this will validate input.
	 *
	 * Note: All sanitized settings will be saved.
	 * Thus, no error messages will be produced.
	 *
	 * Filters in order:
	 * - dbmc_settings_sanitize_<tab_slug>
	 * - dbmc_settings_sanitize_<type>
	 * - dbmc_settings_sanitize
	 * - dbmc_settings_on_change_<tab_slug>
	 * - dbmc_settings_on_change_<field_key>
	 *
	 * @param  array  $input The value submited in the field
	 * @return string        Sanitized value
	 */
	public function settings_sanitize( $input = array() ) {

		if ( empty( $_POST['_wp_http_referer'] ) ) {
		
			return $input;
		
		}

		parse_str( $_POST['_wp_http_referer'], $referrer );
		$tab = isset( $referrer['tab'] ) ? $referrer['tab'] : Dbmc_Settings_Definition::get_default_tab_slug();

		// Tab filter
		$input = apply_filters( 'dbmc_settings_sanitize_' . $tab, $input );

		// Trigger action hook for general settings update for $tab
		$this->do_settings_on_change_hook( $input, $tab );

		// Loop through each setting being saved and pass it through a sanitization filter
		foreach ( $input as $key => $value ) {
	
			$new_value = $value;
			$input[$key] = $this->apply_type_filter( $input, $tab, $key );
			$input[$key] = $this->apply_general_filter( $input, $key );
			$this->do_settings_on_key_change_hook( $key, $new_value );

		}

		add_settings_error( $this->dbmc . '-notices', $this->dbmc, __( 'Settings updated.', $this->dbmc ), 'updated' );

		return $this->get_output( $tab, $input );
	
	}

	/**
	 * Apply the type filter
	 * @param  array   $input Input array
	 * @param  string  $tab   The current tab filters are applied for
	 * @param  string  $key   The key value
	 * @return boolean        True when applied false when failed
	 */
	private function apply_type_filter( $input, $tab, $key ) {

		// Get the setting type (checkbox, select, etc)
		$type = isset( $this->registered_settings[$tab][$key]['type'] ) ? $this->registered_settings[$tab][$key]['type'] : false;

		if ( $type === false ) {
		
			return $input[$key];
		
		}

		return apply_filters( 'dbmc_settings_sanitize_' . $type, $input[$key], $key );

	}

	/**
	 * Genral sanitize filters
	 * @param  array  $input The default array
	 * @param  string $key   The key value for the $input array
	 * @return array         Array containing sanitized filters
	 */
	private function apply_general_filter( $input, $key ) {

		return apply_filters( 'dbmc_settings_sanitize', $input[$key], $key );

	}

	/**
	 * Fired when key changes
	 * @param  string $key       The key value
	 * @param  string $new_value The value to update
	 */
	private function do_settings_on_key_change_hook( $key, $new_value ) {

		$old_plugin_settings = get_option( 'dbmc_settings' );

		if ( isset($old_plugin_settings[$key]) AND $old_plugin_settings[$key] !== $new_value ) {

			do_action( 'dbmc_settings_on_change_' . $key, $new_value, $old_plugin_settings[$key] );

		}

	}

	/**
	 * Tab specific on change hook
	 * @param  array  $new_values The new values for the tab
	 * @param  string $tab        The current tab
	 */
	private function do_settings_on_change_hook( $new_values, $tab ) {

		$old_plugin_settings = get_option( 'dbmc_settings' );
		$changed = false;

		foreach( $new_values as $key => $new_value ) {

			if ( isset($old_plugin_settings[$key]) && $old_plugin_settings[$key] !== $new_value ) {
		
				$changed = true;
		
			}
		
		}

		if ( $changed ) {

			do_action( 'dbmc_settings_on_change_' . $tab, $new_values, $old_plugin_settings );

		}

	}

	/**
	 * Check if value is empty or zero
	 * @param  string  $var The input value
	 * @return boolean      True when empty or zero false if not empty
	 */
	private function not_empty_or_zero( $var ) {

		  return ( !empty( $var ) OR '0' == $var );

	}

	/**
	 * Loop throught the settings and unset empty values
	 * @param  string $tab   The tab being saved
	 * @param  array  $input The input array containing all values
	 * @return array         Final input array, empty values are unset
	 */
	private function get_output( $tab, $input ) {

		$old_plugin_settings = get_option( 'dbmc_settings' );

		if ( ! is_array( $old_plugin_settings ) ) {
		
			$old_plugin_settings = array();
		
		}

		// Remove empty elements
		$input = array_filter( $input, array( $this, 'not_empty_or_zero') );
	
		foreach ( $this->registered_settings[$tab] as $key => $_value ) {

			if ( ! isset( $input[ $key ] ) ) {
	
				$this->do_settings_on_key_change_hook( $key, null );
	
				if( isset( $old_plugin_settings[ $key ] ) ) {
					unset( $old_plugin_settings[ $key ] );
				}
	
			}
	
		}

		// Overwrite the old values with new sanitized ones.
		return array_merge( $old_plugin_settings, $input );
	}

	/**
	 * Sanitize text fields
	 * @param  array  $input The field value
	 * @return string        Sanitized value
	 */
	public function sanitize_text_field( $input ) {

		return sanitize_text_field( $input );

	}

	/**
	 * Sanitize email fields
	 * @param  array  $input The field value
	 * @return string        Sanitized email, return empty string if no email
	 */
	public function sanitize_email_field( $input ) {

		$sanitizes_email = sanitize_email( $input );

		if ( ! is_email( $sanitizes_email ) ) {
	
			$sanitizes_email = __return_empty_string();
	
		}

		return $sanitizes_email;
	
	}

	/**
	 * Sanitize checkbox fields
	 * @param  array  $input The field valye
	 * @return string        '1' if true or empty string when false
	 */
	public function sanitize_checkbox_field( $input ) {

		$true  = array(
			'1',
			'true',
			'True',
			'TRUE',
			'y',
			'Y',
			'yes',
			'Yes',
			'YES',
			'on',
			'On',
			'ON',
			);

		// String
		if ( is_string( $input ) ) {

			$input = trim( $input );

			if ( in_array( $input, $true, true ) ) {
		
				return '1';
		
			}
		
		}

		// Boolean
		if ( is_bool( $input ) AND $input ) {
		
			return '1';
		
		}

		// Integer
		if ( is_int( $input ) AND 1 === $input ) {
		
			return '1';
		
		}

		// Float
		if ( is_float( $input ) AND ! is_nan( $input ) AND (float) 1 === $input ) {
		
			return '1';
		
		}

		return __return_empty_string();
	
	}

	/**
	 * Sanitize url for saving in database
	 * @param  string $input             The url
	 * @param  array  $allowed_protocols Allowed protocols (http, https)
	 * @return string                    URL which is safe to use in database queries
	 */
	public function sanitize_url_field( $input, $allowed_protocols = array( 'http', 'https' ) ) {

		return esc_url_raw( sanitize_text_field( rawurldecode( $input ) ), $allowed_protocols );

	}
	
}
